﻿using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

using Steema.TeeChart;
using System.Diagnostics;

namespace tcharttest
{
    /// <summary>
    /// sample
    /// </summary>
    public partial class Form1 : Form
    {
        #region / Fields                    /

        System.Windows.Forms.Timer timer;
        int currentVertAxisIdx = -1;

        #endregion / Fields                    /
        #region / C'tor                     /

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            InitializeChart();

            timer = new System.Windows.Forms.Timer();
            timer.Interval = 2000; // every 2 seconds
            timer.Tick += new EventHandler(timer_Tick); // switches the current visible custom axis
            /* deactivated
            timer.Enabled = true;
            */
            timer_Tick(null, EventArgs.Empty); // switches ActiveVertAxis to constant line
            timer_Tick(null, EventArgs.Empty); // switches ActiveVertAxis to first ref line

            ForceDraw();
        }

        #endregion / C'tor                     /
        #region / Init                      /
        #region / currently not used        /

        double[] penetrationValues = { -1.0,       0.0,       1.0,
                                        1.0e+014, -1.0e+014,
                                        /* this values lead to chart crashes
                                        1.0e+015, -1.0e+015, 
                                        1.0e+016, -1.0e+016,
                                        */
                                     };

        #endregion / currently not used        /

        Steema.TeeChart.Styles.Line constantLine, referenceLine1, referenceLine2;

        private void InitializeChart()
        {
            tChart1.Aspect.View3D = false;

            tChart1.MouseUp += OnTChartMouseUp;
            tChart1.Zoomed += OnTChartZoomed;
            tChart1.Scroll += OnTChartScroll;

            int samples = 10;

            double val = 1.0e+014; // just add one constant line for now
            {
                constantLine = new Steema.TeeChart.Styles.Line();

                constantLine.Chart = tChart1.Chart;
                constantLine.Color = Color.Red;

                constantLine.VertAxis = Steema.TeeChart.Styles.VerticalAxis.Left;  // has no custom axis and sits on the default left axis

                tChart1.Series.Add(constantLine);

                for (int i = 0; i < samples; i++)
                    constantLine.Add(i, (double)val); // adds a constant signal
            }

            referenceLine1 = AddRefLine(samples, 1, -3, Color.Blue);
            referenceLine2 = AddRefLine(samples, 5, 6, Color.Green);
        }

        private Steema.TeeChart.Styles.Line AddRefLine(int samples, double min, double max, Color color)
        { 
            var referenceLine = new Steema.TeeChart.Styles.Line();
            var referenceAxis = new Steema.TeeChart.Axis();

            referenceLine.Chart = tChart1.Chart;
            referenceLine.Color = color;
            SetAxisColor(referenceAxis, referenceLine.Color);

            referenceLine.CustomVertAxis = referenceAxis;

            referenceAxis.Labels.Style = AxisLabelStyle.Value;

            tChart1.Series.Add(referenceLine);
            tChart1.Axes.Custom.Add(referenceAxis);

            referenceLine.Add(0, min);
            referenceLine.Add(samples - 1, max);

            return referenceLine;
        }

        private void SetAxisColor( Axis axis, Color color )
        {
            axis.AxisPen.Color      = color;
            axis.Ticks.Color        = color;
            axis.Labels.Font.Color  = color;
            axis.Title.Color        = color;
            axis.Title.Font.Color   = color;
        }

        #endregion / Init                      /
        #region / Properties                /

        Axis currentVertAxis = null;
        Axis ActiveVertAxis
        { 
            get
            {
                if (null != currentVertAxis)
                    return currentVertAxis;

                return this.tChart1.Axes.Left;
            }
            set{ currentVertAxis = value; }
        }

        #endregion / Properties                 /
        #region / Timer Handling            /

        void timer_Tick(object sender, EventArgs e)
        { // shuffles through the available axis
            if (tChart1.Axes.Custom.Count > 0)
            {
                foreach (var ax in tChart1.Axes.Custom.OfType<Axis>())
                    ax.Visible = false;

                if (currentVertAxisIdx + 1 < tChart1.Axes.Custom.Count)
                    currentVertAxisIdx++;
                else
                    currentVertAxisIdx = 0;

                Axis currentAxis = tChart1.Axes.Custom[currentVertAxisIdx];
                currentAxis.Visible = true;
                ActiveVertAxis = currentAxis;

                ForceDraw();
            }
        }

        #endregion / Timer Handling        /
        #region / Zoom Handling             /

        private Point m_MouseDownLocation;
        private Point m_MouseUpLocation;

        double bottomAxisMin = 0;
        double bottomAxisMax = 0;

        /// <summary>
        /// Called when TeeChart mouse up event occurs.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        protected void OnTChartMouseUp(object sender, MouseEventArgs e)
        {
            if (tChart1.Zoom.Active)
            {
                m_MouseDownLocation.X = tChart1.Zoom.x0;
                m_MouseDownLocation.Y = tChart1.Zoom.y0;
                m_MouseUpLocation.X = tChart1.Zoom.x1;
                m_MouseUpLocation.Y = tChart1.Zoom.y1;
            }

            bottomAxisMin = tChart1.Axes.Bottom.Minimum;
            bottomAxisMax = tChart1.Axes.Bottom.Maximum;
        }

        /// <summary>
        /// Event handler, gets called when TeeChart shows a new zooming state
        /// </summary>
        /// <param name="sender">should be the chart</param>
        /// <param name="e">info about the zoom</param>
        ///           ____________________
        internal void OnTChartZoomed(object sender, EventArgs e)
        {
            //SCALINGAXES:
            //According to steemasupportfaq we have to consider the zoom here ourselves...

            if (tChart1.Zoom.Direction == ZoomDirections.Vertical)
            { // workaround for AP01407865 : see http://www.teechart.net/support/viewtopic.php?f=4&t=1897&p=6383&hilit=ZoomDirections#p6383
                tChart1.Axes.Bottom.SetMinMax( // reset Bottom axis, as Steema modifies it, even there is only a vertical zoom
                    bottomAxisMin,
                    bottomAxisMax);
            }

            if (tChart1.Zoom.Direction == ZoomDirections.Both ||    // only zoom the custom vert axis if vertical zoom direction is included
                tChart1.Zoom.Direction == ZoomDirections.Vertical)
            {
                foreach (var ax in tChart1.Axes.Custom.OfType<Axis>())
                {
                    double min = ax.CalcPosPoint(m_MouseUpLocation.Y);
                    double max = ax.CalcPosPoint(m_MouseDownLocation.Y);

                    ax.SetMinMax(min, max);
                }
            }
        }

        #endregion / Zoom Handling             /
        #region / Scroll Handling           /

        void OnTChartScroll(object sender, EventArgs e)
        {
            //foreach (var series in tChart1.Series.OfType<Steema.TeeChart.Styles.Series>().Where(
            //    victim => victim.CustomVertAxis != null))
            //    series.CustomVertAxis.SetMinMax(series.YValues.Minimum, series.YValues.Maximum);

            if (!this.checkBox1.Checked)
            {
                this.tChart1.Axes.Left.SetMinMax(constantLine.YValues.Minimum, constantLine.YValues.Maximum);
            }

            if (!this.checkBox2.Checked)
            {
                referenceLine1.CustomVertAxis.SetMinMax(referenceLine1.YValues.Minimum, referenceLine1.YValues.Maximum);
            }
            if (!this.checkBox3.Checked)
            {
                referenceLine2.CustomVertAxis.SetMinMax(referenceLine2.YValues.Minimum, referenceLine2.YValues.Maximum);
            }
        }

        #endregion / Scroll Handling           /
        #region / Layout Adjustments        /

        internal void ForceDraw()
        {
            //TeeChart got a bug regarding internalDraw when window is not visible (e.g. the window is to small).
            // in this case a redraw is not needed as nothing is been seen anyhow and as soon as the window gets resised
            //the draw accours anyways
            try
            {
                if (chartHorizontalScrollPanel.Visible &&
                    (tChart1.Height > 1 && tChart1.Width > 1))
                {
                    tChart1.Invalidate(true);
                    CalcLeftAxisWidth();
                    tChart1.Draw();
                }
            }
            catch (Exception)
            {
                return;
            }
        }

        const int tchartsPanelMarginLeftOffset = 12; // in pixel
        private void CalcLeftAxisWidth()
        {
            tChart1.Panel.MarginLeft = ActiveVertAxis.MaxLabelsWidth() + tchartsPanelMarginLeftOffset;
        }

        #endregion / Layout Adjustments        /
        #region / Button Handling           /

        private void tChart1ShwoEditorButton_Click( object sender, EventArgs e )
        {
            tChart1.ShowEditor();
        }

        #endregion / Button Handling        /
    }
}